@@ -1,5 +1,24 @@
 from datetime import date, datetime as dt, timedelta as td
-from datetime import datetime
+from dateutil.relativedelta import relativedelta
+
+
+class DateUts():
+    date = None
+
+    def __init__(self,date):
+        self.date = date
+
+    def is_weekend(self):
+        return self.date.weekday() in [5, 6]
+    
+    def weekday(self):
+        return self.date.weekday()
+
+    def __repr__(self):
+        return f"<DateUts {self.date.strftime('%Y-%m-%d %H:%M:%S')}>"
+
+
+# output: True (2023-04-22 is a Saturday)
 
 
 #========= USAGE ============
@@ -7,8 +26,8 @@ from datetime import datetime
 # > sqlToDate('yyyy-MM-dd')
 # > <datetime>
 
-def sqlToDate(dt:str):
-    return datetime.strptime(dt,"%Y-%m-%d")
+def sqlToDate(date_str:str):
+    return DateUts(dt.strptime(date_str,"%Y-%m-%d"))
 
 #========= USAGE ============
 #Ex1:
@@ -16,7 +35,7 @@ def sqlToDate(dt:str):
 # > 'yyyy-MM-dd'
 
 def dateToSql(dt:date):
-    return datetime.strftime(dt,"%Y-%m-%d")
+    return DateUts(dt.strftime(dt,"%Y-%m-%d"))
 
 #========= USAGE ============
 #Ex1:
@@ -33,7 +52,7 @@ def now(fmt=None):
 # > <datetime>, 'yyyy-MM-dd',  'yyyy-MM-dd'
 
 def today(fmt=None,addDays=0):
-    v =  dt.now() 
+    v =  dt.now().date()
     v =  v if not addDays else dateAdd(today(),addDays,'day')
     return fmtDate(v,fmt)
 
@@ -43,7 +62,11 @@ def today(fmt=None,addDays=0):
 # > <datetime>, 'yyyy-MM-dd',  'yyyy-MM-dd'
 
 def yesterday(fmt=None):
-    v = dt.now() - td(1)
+    v = today().date - td(1)
+    return fmtDate(v,fmt)
+
+def tomorrow(fmt=None):
+    v = today().date + td(1)
     return fmtDate(v,fmt)
 
 #========= USAGE ============
@@ -53,6 +76,9 @@ def yesterday(fmt=None):
 # > [<datetime>,<datetime>], ['2022-05-23','2022-05-24'],  ['2022-05-23','2022-05-24']
 
 def dateRange(start:date,end:date,fmt=None,filter_lbd:callable=None):
+    start = start.date if isinstance(start,DateUts) else start
+    end   = end.date if isinstance(end,DateUts) else end
+
     if start > end:
         dates = [dateAdd(start,x*-1) for x in range(0, (start-end).days + 1)]
     else:
@@ -74,10 +100,21 @@ def dateRange(start:date,end:date,fmt=None,filter_lbd:callable=None):
 # > <datetime>, '2022-05-22',  '2022-05-22'
 
 def dateAdd(date:date,qtd:int,unit:str="day",fmt=None):
+    date = date.date if isinstance(date,DateUts) else date
     if unit == 'day':
         v = date + td(qtd) if qtd > 0 else date - td(abs(qtd))
     elif unit == 'year':
         v = date.replace(year = date.year + qtd)
+    elif unit == 'hour':
+        v = date + td(hours=qtd)
+    elif unit == 'minute':
+        v = date + td(minutes=qtd)
+    elif unit == 'second':
+        v = date + td(seconds=qtd)
+    elif unit == 'month':
+        v = date + relativedelta(months=qtd)
+
+
 
     return fmtDate(v,fmt)
 
@@ -96,6 +133,15 @@ def lastWorkingDate(ref:date=None,fmt=None): #IGNORE SATURDAY AND SUNDAY
         return lastWorkingDate(y,fmt)
     return fmtDate(y,fmt)
 
+
+
+def nextWorkingDate(ref:date=None,fmt=None): #IGNORE SATURDAY AND SUNDAY
+    t = tomorrow() if not ref else dateAdd(ref,1,'day')
+    if t.weekday() in [5,6]:
+        return nextWorkingDate(t,fmt)
+    return fmtDate(t,fmt)
+
+
 #========= USAGE ============
 #Ex1:
 # > today()  ,  today(fmt='%Y-%m-%d')   ,   today(fmt='sql')
@@ -103,7 +149,9 @@ def lastWorkingDate(ref:date=None,fmt=None): #IGNORE SATURDAY AND SUNDAY
 
 def fmtDate(dt:date,fmt:str):
     fmt= fmt if not fmt else ("%Y-%m-%d" if fmt == "sql" else fmt)
-    return dt if not fmt else dt.strftime(fmt)
+    dt = dt.date if isinstance(dt,DateUts) else dt
+
+    return DateUts(dt) if not fmt else dt.strftime(fmt)
 
 def dateMatch(dt:str,fmt:str):
     fmt = "%Y-%m-%d" if fmt == "sql" else fmt
@@ -115,7 +163,26 @@ def dateMatch(dt:str,fmt:str):
 
     return True
 
+def firstDay(sql_dte:str|date,fmt:str=None):
+    dte = sql_dte if isinstance(sql_dte,date) or isinstance(sql_dte,DateUts) else sqlToDate(sql_dte)
+    dte = fmtDate(dte,"%Y-%m-01")
+    firstDay = sqlToDate(dte)
+    return fmtDate(firstDay,fmt)
 
+def lastDay(sql_dte:str|date,fmt:str=None):
+    dte = sql_dte if isinstance(sql_dte,date) or isinstance(sql_dte,DateUts) else sqlToDate(sql_dte)
+    dte = sqlToDate(dateAdd(dte,1,"month",fmt="%Y-%m-01"))
+    dte = dateAdd(dte,-1,"day")
+    return fmtDate(dte,fmt)
+
+
+
+Fnc_noWeekends = lambda dt:dt.weekday() not in [5,6]
+
+#a = dateRange(firstDay(today()).date,lastDay(today()).date,filter_lbd = Fnc_noWeekends)
+#a[0].weekday()
+# a = DateUts(dt.now())
+# print(a)
 # a = lastWorkingDate(fmt="%Y-%m-%d")
 # rng = dateRange(sqlToDate("2022-05-01"),sqlToDate("2022-05-10"))
 # rng = dateRange(sqlToDate("2022-05-10"),sqlToDate("2022-05-01"))
